@@ -142,6 +142,7 @@ typedef struct {
   char *cmml_dir;         /**< dir attribute of cmml tag */
   char *cmml_id;          /**< id attribute of cmml tag */
   char *cmml_xmlns;       /**< xmlns attribute of cmml tag */
+  char *cmml_granulerate; /**< granulerate attribute of cmml tag */
 } CMML_Preamble;
 
 
@@ -180,7 +181,7 @@ typedef struct {
  */
 typedef struct {
   char *id;             /**< id attribute of stream element */
-  CMML_Time * timebase; /**< timebase attribute of stream element */
+  CMML_Time * basetime; /**< basetime attribute of stream element */
   CMML_Time * utc;      /**< utc attribute of stream element */
   CMML_List * import;   /**< list of import elements */
 } CMML_Stream;
@@ -199,6 +200,23 @@ typedef struct {
   char *scheme;  /**< scheme name of meta element */
 } CMML_MetaElement;
 
+/**
+ * CMML_LinkElement: a struct to keep the details of the link tag in
+ * either a head or a clip
+ */
+typedef struct {
+  char *id;      /**< id attribute of link */
+  char *class;   /**< class attribute of link */
+  char *title;   /**< title attribute of link */
+  char *lang;    /**< language code of link */
+  char *dir;     /**< directionality of lang (ltr/rtl) */
+  char *href;    /**< href attribute of link */
+  char *type;    /**< type attribute of link */
+  char *rel;     /**< rel attribute of link */
+  char *rev;     /**< rev attribute of link */
+  char *media;   /**< media attribute of link */
+} CMML_LinkElement;
+
 /**
  * CMML_Head: a head element as presented in the CMML document.
  */
@@ -214,6 +232,7 @@ typedef struct {
   char *base_id;    /**< id attribute of base element */
   char *base_href;  /**< href attribute of base element */
   CMML_List * meta; /**< list of meta elements */
+  CMML_List * link; /**< list of link elements */
 } CMML_Head;
 
 
@@ -222,6 +241,8 @@ typedef struct {
  */
 typedef struct {
   char *clip_id;         /**< id attribute of clip */
+  char *class;           /**< class attribute of clip */
+  char *title;           /**< title attribute of clip */
   char *lang;            /**< language attribute of desc element */
   char *dir;             /**< directionality of lang */
   char *track;           /**< track attribute of clip */
@@ -230,16 +251,21 @@ typedef struct {
   CMML_List * meta;      /**< list of meta elements */
   char *anchor_id;       /**< id attribute of anchor */
   char *anchor_class;    /**< class attribute of clip */
+  char *anchor_title;    /**< title attribute of clip */
   char *anchor_lang;     /**< language of anchor */
   char *anchor_dir;      /**< directionality of lang */
   char *anchor_href;     /**< href out of clip */
   char *anchor_text;     /**< anchor text */
   char *img_id;          /**< id attribute of image */
+  char *img_class;       /**< class attribute of image */
+  char *img_title;       /**< title attribute of image */
   char *img_lang;        /**< language of img */
   char *img_dir;         /**< directionality of lang */
   char *img_src;         /**< keyframe image of clip */
   char *img_alt;         /**< alternate text for image */
   char *desc_id;         /**< id attribute of desc */
+  char *desc_class;      /**< class attribute of desc */
+  char *desc_title;      /**< title attribute of desc */
   char *desc_lang;       /**< language attribute of desc element */
   char *desc_dir;        /**< directionality of lang */
   char *desc_text;       /**< the description itself */
@@ -594,10 +620,11 @@ double cmml_skip_to_offset (CMML *cmml, const char * offset);
  * \param id the id attribute of the cmml tag; not used if NULL
  * \param lang the lang attribute of the cmml tag; not used if NULL
  * \param dir the dir attribute of the cmml tag; not used if NULL
+ * \param granulerate the granulerate attribute of the cmml tag; not used if NULL
  *
  * \returns a CMML preamble
  */
-CMML_Preamble * cmml_preamble_new (char *encoding, char *id, char *lang, char *dir);
+CMML_Preamble * cmml_preamble_new (char *encoding, char *id, char *lang, char *dir, char *granulerate);
 
 /**
  * cmml_element_new: Create a new CMML_Element.
@@ -1021,6 +1048,15 @@ CMML_Time * cmml_time_new  (const char * s);
  */
 CMML_Time * cmml_sec_new   (const char * s);
 
+/**
+ * cmml_time_new_sec: Creates a time struct from a npt sec specs.
+ *
+ * \param seconds sec spec to be used for creating a new time struct
+ *
+ * \returns a time struct.
+ */
+CMML_Time * cmml_time_new_secs (double seconds);
+
 /**
  * cmml_utc_new: Creates a time struct only from name-value utc specs.
  *
@@ -1053,7 +1089,7 @@ int cmml_time_interval_new (const char * s,
  *
  * \param s the string that contains the time spec
  * \param ref the reference utc time if given in the stream tag
- * \param base the timebase in seconds as given in the stream tag
+ * \param base the basetime in seconds as given in the stream tag
  *
  * \returns a time structure in seconds
  */
@@ -1061,11 +1097,11 @@ CMML_Time *cmml_time_new_in_sec (const char * s, CMML_Time * ref, double base);
 
 /**
  * cmml_time_utc_to_sec: Convert a time from utc to seconds with
- * reference to a timebase
+ * reference to a basetime
  *
  * \param t the given time struct
  * \param ref the reference utc time if given in the stream tag
- * \param base the timebase in seconds as given in the stream tag
+ * \param base the basetime in seconds as given in the stream tag
  *
  * \returns a time structure in seconds
  */
