@@ -54,7 +54,7 @@
  * cmml_preamble_new: Create a new CMML_Preamble.
  */
 CMML_Preamble * 
-cmml_preamble_new (char *encoding, char *id, char *lang, char *dir)
+cmml_preamble_new (char *encoding, char *id, char *lang, char *dir, char *granulerate)
 {
   CMML_Preamble * p;
   p = (CMML_Preamble *) cmml_malloc(sizeof(CMML_Preamble));
@@ -81,6 +81,11 @@ cmml_preamble_new (char *encoding, char *id, char *lang, char *dir)
   } else {
     p->cmml_id = NULL;
   }
+  if (granulerate) {
+    p->cmml_granulerate = cmml_strdup(granulerate);
+  } else {
+    p->cmml_granulerate = NULL;
+  }
   p->cmml_xmlns = cmml_strdup("http://www.annodex.net/cmml");
   return p;
 }
@@ -119,7 +124,7 @@ cmml_stream_new ()
   CMML_Stream * s;
   s = (CMML_Stream *) cmml_malloc(sizeof(CMML_Stream));
   s->id = NULL;
-  s->timebase = NULL;
+  s->basetime = NULL;
   s->utc = NULL;
   s->import = cmml_list_new();
   return s;
@@ -141,6 +146,7 @@ cmml_head_new    (void)
   h->base_id = NULL;
   h->base_href = NULL;
   h->meta = cmml_list_new();
+  h->link = cmml_list_new();
   return h;
 }
 
@@ -150,6 +156,8 @@ cmml_clip_new (CMML_Time *start_time, CMML_Time *end_time)
   CMML_Clip * a;
   a = (CMML_Clip *) cmml_malloc(sizeof(CMML_Clip));
   a->clip_id      = NULL;
+  a->class        = NULL;
+  a->title        = NULL;
   a->lang         = NULL;
   a->dir          = NULL;
   a->track        = NULL;
@@ -158,16 +166,21 @@ cmml_clip_new (CMML_Time *start_time, CMML_Time *end_time)
   a->meta         = cmml_list_new();
   a->anchor_id    = NULL;
   a->anchor_class = NULL;
+  a->anchor_title = NULL;
   a->anchor_lang  = NULL;
   a->anchor_dir   = NULL;
   a->anchor_href  = NULL;
   a->anchor_text  = NULL;
   a->img_id       = NULL;
+  a->img_class    = NULL;
+  a->img_title    = NULL;
   a->img_lang     = NULL;
   a->img_dir      = NULL;
   a->img_src      = NULL;
   a->img_alt      = NULL;
   a->desc_id      = NULL;
+  a->desc_class   = NULL;
+  a->desc_title   = NULL;
   a->desc_lang    = NULL;
   a->desc_dir     = NULL;
   a->desc_text    = NULL;
@@ -248,6 +261,29 @@ cmml_meta_element_clone (CMML_MetaElement * meta)
   return new_meta;
 }
 
+static CMML_LinkElement *
+cmml_link_element_clone (CMML_LinkElement * link)
+{
+  CMML_LinkElement * new_link;
+
+  if (link == NULL) return NULL;
+
+  new_link = (CMML_LinkElement* ) cmml_malloc (sizeof(CMML_LinkElement));
+  /* maybe consider memcpy for speedup and introduce reference counting!
+     memcpy (new_link, link, sizeof (*new_link));
+  */
+  new_link->id      = cmml_strdup (link->id);
+  new_link->class   = cmml_strdup (link->class);
+  new_link->title   = cmml_strdup (link->title);
+  new_link->lang    = cmml_strdup (link->lang);
+  new_link->dir     = cmml_strdup (link->dir);
+  new_link->href    = cmml_strdup (link->href);
+  new_link->type    = cmml_strdup (link->type);
+  new_link->rel     = cmml_strdup (link->rel);
+  new_link->rev     = cmml_strdup (link->rev);
+  new_link->media   = cmml_strdup (link->media);
+  return new_link;
+}
 
 CMML_Preamble * 
 cmml_preamble_clone (CMML_Preamble * src)
@@ -263,6 +299,7 @@ cmml_preamble_clone (CMML_Preamble * src)
   p->cmml_dir         = cmml_strdup(src->cmml_dir);
   p->cmml_id          = cmml_strdup(src->cmml_id);
   p->cmml_xmlns       = cmml_strdup(src->cmml_xmlns);
+  p->cmml_granulerate = cmml_strdup(src->cmml_granulerate);
   return p;
 }
 
@@ -303,7 +340,7 @@ cmml_stream_clone(CMML_Stream * src)
   memcpy (s, src, sizeof (*s));
   */
   s->id       = cmml_strdup(src->id);
-  s->timebase = cmml_time_clone(src->timebase);
+  s->basetime = cmml_time_clone(src->basetime);
   if (src->utc != NULL) {
     s->utc      = cmml_time_clone(src->utc);
   } else {
@@ -336,6 +373,8 @@ cmml_head_clone(CMML_Head * head)
   new_head->base_href  = cmml_strdup(head->base_href);
   new_head->meta =
     cmml_list_clone_with (head->meta, (CMML_CloneFunc)cmml_meta_element_clone);
+  new_head->link =
+    cmml_list_clone_with (head->link, (CMML_CloneFunc)cmml_link_element_clone);
   return new_head;
 }
 
@@ -349,6 +388,8 @@ cmml_clip_clone (CMML_Clip * src)
   memcpy (a, src, sizeof (*a));
   */
   a->clip_id    = cmml_strdup(src->clip_id);
+  a->class      = cmml_strdup(src->class);
+  a->title      = cmml_strdup(src->title);
   a->lang       = cmml_strdup(src->lang);
   a->dir        = cmml_strdup(src->dir);
   a->track      = cmml_strdup(src->track);
@@ -358,16 +399,21 @@ cmml_clip_clone (CMML_Clip * src)
     cmml_list_clone_with (src->meta, (CMML_CloneFunc)cmml_meta_element_clone);
   a->anchor_id       = cmml_strdup(src->anchor_id);
   a->anchor_class    = cmml_strdup(src->anchor_class);
+  a->anchor_title    = cmml_strdup(src->anchor_title);
   a->anchor_lang     = cmml_strdup(src->anchor_lang);
   a->anchor_dir      = cmml_strdup(src->anchor_dir);
   a->anchor_href     = cmml_strdup(src->anchor_href);
   a->anchor_text     = cmml_strdup(src->anchor_text);
   a->img_id       = cmml_strdup(src->img_id);
+  a->img_class    = cmml_strdup(src->img_class);
+  a->img_title    = cmml_strdup(src->img_title);
   a->img_lang     = cmml_strdup(src->img_lang);
   a->img_dir      = cmml_strdup(src->img_dir);
   a->img_src      = cmml_strdup(src->img_src);
   a->img_alt      = cmml_strdup(src->img_alt);
   a->desc_id      = cmml_strdup(src->desc_id);
+  a->desc_class   = cmml_strdup(src->desc_class);
+  a->desc_title   = cmml_strdup(src->desc_title);
   a->desc_lang    = cmml_strdup(src->desc_lang);
   a->desc_dir     = cmml_strdup(src->desc_dir);
   a->desc_text    = cmml_strdup(src->desc_text);
@@ -420,6 +466,23 @@ cmml_meta_element_destroy (CMML_MetaElement * meta)
   return NULL;
 }
 
+CMML_LinkElement *
+cmml_link_element_destroy (CMML_LinkElement * link)
+{
+  if (link == NULL) return NULL;
+  cmml_free (link->id);
+  cmml_free (link->class);
+  cmml_free (link->title);
+  cmml_free (link->lang);
+  cmml_free (link->dir);
+  cmml_free (link->href);
+  cmml_free (link->type);
+  cmml_free (link->rel);
+  cmml_free (link->rev);
+  cmml_free (link->media);
+  return NULL;
+}
+
 void
 cmml_preamble_destroy(CMML_Preamble * info)
 {
@@ -430,6 +493,7 @@ cmml_preamble_destroy(CMML_Preamble * info)
   cmml_free(info->cmml_dir);
   cmml_free(info->cmml_id);
   cmml_free(info->cmml_xmlns);
+  cmml_free(info->cmml_granulerate);
   cmml_free(info);
   return;
 }
@@ -465,7 +529,7 @@ cmml_stream_destroy(CMML_Stream * s)
 {
   if (s == NULL) return;
   cmml_free(s->id);
-  cmml_time_free(s->timebase);
+  cmml_time_free(s->basetime);
   cmml_time_free(s->utc);
   cmml_list_free_with(s->import, (CMML_FreeFunc) cmml_import_element_destroy);
   cmml_free(s);
@@ -490,6 +554,7 @@ cmml_head_destroy(CMML_Head * h)
   cmml_free(h->base_id);
   cmml_free(h->base_href);
   cmml_list_free_with(h->meta, (CMML_FreeFunc) cmml_meta_element_destroy);
+  cmml_list_free_with(h->link, (CMML_FreeFunc) cmml_link_element_destroy);
   cmml_free(h);
   return;
 }
@@ -502,6 +567,8 @@ cmml_clip_destroy(CMML_Clip * a)
 {
   if (a == NULL) return;
   cmml_free(a->clip_id);
+  cmml_free(a->class);
+  cmml_free(a->title);
   cmml_free(a->lang);
   cmml_free(a->dir);
   cmml_free(a->track);
@@ -510,16 +577,21 @@ cmml_clip_destroy(CMML_Clip * a)
   cmml_list_free_with(a->meta, (CMML_FreeFunc) cmml_meta_element_destroy);
   cmml_free(a->anchor_id);
   cmml_free(a->anchor_class);
+  cmml_free(a->anchor_title);
   cmml_free(a->anchor_lang);
   cmml_free(a->anchor_dir);
   cmml_free(a->anchor_href);
   cmml_free(a->anchor_text);
   cmml_free(a->img_id);
+  cmml_free(a->img_class);
+  cmml_free(a->img_title);
   cmml_free(a->img_lang);
   cmml_free(a->img_dir);
   cmml_free(a->img_src);
   cmml_free(a->img_alt);
   cmml_free(a->desc_id);
+  cmml_free(a->desc_class);
+  cmml_free(a->desc_title);
   cmml_free(a->desc_lang);
   cmml_free(a->desc_dir);
   cmml_free(a->desc_text);
@@ -652,6 +724,43 @@ int cmml_meta_list_snprint (char *buf, int *_len, int *_n, int *_n_written,
   return 0;
 }
 
+/* return number of characters written */
+static
+int cmml_link_list_snprint (char *buf, int *_len, int *_n, int *_n_written, 
+			    CMML_List * link)
+{
+  int len = *_len;
+  int n = *_n;
+  int n_written = *_n_written;
+  
+  if (!cmml_list_is_empty(link)) {
+    CMML_List * link_node;
+      
+    for (link_node = link;
+	 link_node != NULL;
+	 link_node = link_node->next) {
+      CMML_LinkElement * l = (CMML_LinkElement *) link_node->data;
+	
+      BEGIN_TAG("link");
+      APPEND_ATTRIBUTE("id", l->id);
+      APPEND_ATTRIBUTE("class", l->class);
+      APPEND_ATTRIBUTE("title", l->title);
+      APPEND_ATTRIBUTE("lang", l->lang);
+      APPEND_ATTRIBUTE("dir", l->dir);
+      APPEND_ATTRIBUTE("href", l->href);
+      APPEND_ATTRIBUTE("type", l->type);
+      APPEND_ATTRIBUTE("rel", l->rel);
+      APPEND_ATTRIBUTE("rev", l->rev);
+      APPEND_ATTRIBUTE("media", l->media);
+      END_EMPTY_TAG();
+      NEWLINE;
+    }
+  }
+  *_len = len;
+  *_n = n;
+  *_n_written = n_written;
+  return 0;
+}
 
 int
 cmml_preamble_snprint(char *buf, int n, CMML_Preamble *pre)
@@ -698,6 +807,7 @@ cmml_preamble_snprint(char *buf, int n, CMML_Preamble *pre)
   /* to print or not to print?
   APPEND_ATTRIBUTE("xmlns", pre->cmml_xmlns);
   */
+  APPEND_ATTRIBUTE("granulerate", pre->cmml_granulerate);
   END_TAG();
   NEWLINE;
 
@@ -750,8 +860,8 @@ cmml_stream_snprint(char *buf, int n, CMML_Stream * m)
   
   BEGIN_TAG("stream");
   APPEND_ATTRIBUTE("id", m->id);
-  if (m->timebase) {
-    APPEND_ATTRIBUTE("timebase", m->timebase->tstr);
+  if (m->basetime) {
+    APPEND_ATTRIBUTE("basetime", m->basetime->tstr);
   }
   if (m->utc) {
     APPEND_ATTRIBUTE("utc", m->utc->tstr);
@@ -775,8 +885,8 @@ cmml_stream_pretty_snprint(char *buf, int n, CMML_Stream * m)
   
   BEGIN_TAG("stream");
   APPEND_ATTRIBUTE("id", m->id);
-  if (m->timebase) {
-    APPEND_ATTRIBUTE("timebase", m->timebase->tstr);
+  if (m->basetime) {
+    APPEND_ATTRIBUTE("basetime", m->basetime->tstr);
   }
   if (m->utc) {
     APPEND_ATTRIBUTE("utc", m->utc->tstr);
@@ -868,6 +978,8 @@ cmml_head_pretty_snprint(char *buf, int n, CMML_Head * h)
 
   if (cmml_meta_list_snprint(buf, &len, &n, &n_written, h->meta) < 0)
     return -1;
+  if (cmml_link_list_snprint(buf, &len, &n, &n_written, h->link) < 0)
+    return -1;
   CLOSE_TAG("head");
   NEWLINE;
 
@@ -883,6 +995,8 @@ int cmml_clip_snprint(char *buf, int n, CMML_Clip * a)
   
   BEGIN_TAG("clip");
   APPEND_ATTRIBUTE("id", a->clip_id);
+  APPEND_ATTRIBUTE("class", a->class);
+  APPEND_ATTRIBUTE("title", a->title);
   APPEND_ATTRIBUTE("lang", a->lang);
   APPEND_ATTRIBUTE("dir", a->dir);
   APPEND_ATTRIBUTE("track", a->track);
@@ -898,7 +1012,8 @@ int cmml_clip_snprint(char *buf, int n, CMML_Clip * a)
   if (a->anchor_id || a->anchor_text || a->anchor_href) {
     BEGIN_TAG("a");
     APPEND_ATTRIBUTE("id",    a->anchor_id);
-    APPEND_ATTRIBUTE("class", a->anchor_class);
+    APPEND_ATTRIBUTE("class",  a->anchor_class);
+    APPEND_ATTRIBUTE("title",   a->anchor_title);
     APPEND_ATTRIBUTE("lang",  a->anchor_lang);
     APPEND_ATTRIBUTE("dir",   a->anchor_dir);
     APPEND_ATTRIBUTE("href",  a->anchor_href);
@@ -911,6 +1026,8 @@ int cmml_clip_snprint(char *buf, int n, CMML_Clip * a)
   if (a->img_id || a->img_src || a->img_alt) {
     BEGIN_TAG("img");
     APPEND_ATTRIBUTE("id",   a->img_id);
+    APPEND_ATTRIBUTE("class", a->img_class);
+    APPEND_ATTRIBUTE("title",  a->img_title);
     APPEND_ATTRIBUTE("lang", a->img_lang);
     APPEND_ATTRIBUTE("dir",  a->img_dir);
     APPEND_ATTRIBUTE("src",  a->img_src);
@@ -922,6 +1039,8 @@ int cmml_clip_snprint(char *buf, int n, CMML_Clip * a)
   if (a->desc_id || a->desc_text) {
     BEGIN_TAG("desc");
     APPEND_ATTRIBUTE("id",   a->desc_id);
+    APPEND_ATTRIBUTE("class", a->desc_class);
+    APPEND_ATTRIBUTE("title",  a->desc_title);
     APPEND_ATTRIBUTE("lang", a->desc_lang);
     APPEND_ATTRIBUTE("dir",  a->desc_dir);
     END_TAG();
@@ -942,6 +1061,8 @@ int cmml_clip_pretty_snprint(char *buf, int n, CMML_Clip * a)
   
   BEGIN_TAG("clip");
   APPEND_ATTRIBUTE("id", a->clip_id);
+  APPEND_ATTRIBUTE("class", a->class);
+  APPEND_ATTRIBUTE("title", a->title);
   APPEND_ATTRIBUTE("lang", a->lang);
   APPEND_ATTRIBUTE("dir", a->dir);
   APPEND_ATTRIBUTE("track", a->track);
@@ -959,6 +1080,7 @@ int cmml_clip_pretty_snprint(char *buf, int n, CMML_Clip * a)
     BEGIN_TAG("a");
     APPEND_ATTRIBUTE("id", a->anchor_id);
     APPEND_ATTRIBUTE("class", a->anchor_class);
+    APPEND_ATTRIBUTE("title", a->anchor_title);
     APPEND_ATTRIBUTE("lang", a->anchor_lang);
     APPEND_ATTRIBUTE("dir", a->anchor_dir);
     APPEND_ATTRIBUTE("href", a->anchor_href);
@@ -972,6 +1094,8 @@ int cmml_clip_pretty_snprint(char *buf, int n, CMML_Clip * a)
   if (a->img_id || a->img_src || a->img_alt) {
     BEGIN_TAG("img");
     APPEND_ATTRIBUTE("id", a->img_id);
+    APPEND_ATTRIBUTE("class", a->img_class);
+    APPEND_ATTRIBUTE("title", a->img_title);
     APPEND_ATTRIBUTE("lang", a->img_lang);
     APPEND_ATTRIBUTE("dir", a->img_dir);
     APPEND_ATTRIBUTE("src", a->img_src);
@@ -984,6 +1108,8 @@ int cmml_clip_pretty_snprint(char *buf, int n, CMML_Clip * a)
   if (a->desc_id || a->desc_text) {
     BEGIN_TAG("desc");
     APPEND_ATTRIBUTE("id", a->desc_id);
+    APPEND_ATTRIBUTE("class", a->desc_class);
+    APPEND_ATTRIBUTE("title", a->desc_title);
     APPEND_ATTRIBUTE("lang", a->desc_lang);
     APPEND_ATTRIBUTE("dir", a->desc_dir);
     END_TAG();
