@@ -48,7 +48,7 @@
 
 /*
  * cmml_parse_to_head: parse the top bit of the cmml file until it
- * reaches the head tag to get all the timebase etc. information
+ * reaches the head tag to get all the basetime etc. information
  */
 int
 cmml_parse_to_head (CMML_Context * ctxt)
@@ -58,10 +58,10 @@ cmml_parse_to_head (CMML_Context * ctxt)
   long bytes, nread = 0;
 
   /* if stream tag had not previously been read, read until you've got it by */
-  /* checking for timebase as this is a stream attribute that is obligatory */
+  /* checking for basetime as this is a stream attribute that is obligatory */
   /* or until you have a head tag */
   while (ctxt->status->stream == NULL || 
-	 ctxt->status->stream->timebase == NULL) {
+	 ctxt->status->stream->basetime == NULL) {
 	  if ((bytes = (long)fread(buf, 1, READ_BUF_SIZE, ctxt->file)) == -1) {
       cmml_set_error((CMML*)ctxt, CMML_READ_ERROR);
       return -1;
@@ -153,6 +153,7 @@ static int
 insert_stream_attrs(CMML_Stream * stream, const XML_Char **attr, CMML_Context *ctxt)
 {
   int i;
+  CMML_Time * timebase = NULL; /* backwards compatibility for CMML 2.1 */
 
   if (!stream) return 0;
 
@@ -162,8 +163,10 @@ insert_stream_attrs(CMML_Stream * stream, const XML_Char **attr, CMML_Context *c
 #endif
     if (cmml_strcmp(attr[i], "id") == 0) {
       stream->id = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "basetime") == 0) {
+      stream->basetime = cmml_time_new (attr[i+1]);
     } else if (cmml_strcmp(attr[i], "timebase") == 0) {
-      stream->timebase = cmml_time_new (attr[i+1]);
+      timebase = cmml_time_new(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "utc") == 0) {
       stream->utc = cmml_utc_new (attr[i+1]);
       if (!stream->utc->t.utc) { /* just ignoring tag */
@@ -174,12 +177,20 @@ insert_stream_attrs(CMML_Stream * stream, const XML_Char **attr, CMML_Context *c
     }
   }
   /* adapt timebase */
-  if (stream->timebase == NULL) {
-    /* default value */
-    stream->timebase = cmml_time_new("0");
+  if (stream->basetime == NULL) {
+    if (timebase != NULL) {
+      stream->basetime = timebase;
+    } else {
+      /* default value */
+      stream->basetime = cmml_time_new("0");
+    }
+  } else { /* timebase is not being used */
+    if (timebase != NULL) {
+      cmml_time_free (timebase);
+    }
   }
-  /* check parsed timebase as it is used later in library */
-  if (stream->timebase->t.sec < 0.0) {
+  /* check parsed basetime as it is used later in library */
+  if (stream->basetime->t.sec < 0.0) {
     cmml_set_error((CMML*)ctxt, CMML_TIME_ERROR);
     return -1; /* stream to be destroyed */
   }
@@ -229,12 +240,12 @@ insert_import_tag (CMML_List * import_list, const XML_Char **attr, CMML_Context
       m->start_time = 
 	cmml_time_new_in_sec (attr[i+1],
 			      ctxt->status->stream->utc,
-			      ctxt->status->stream->timebase->t.sec);
+			      ctxt->status->stream->basetime->t.sec);
     } else if (cmml_strcmp(attr[i], "end") == 0) {
       m->end_time = 
 	cmml_time_new_in_sec (attr[i+1],
 			      ctxt->status->stream->utc,
-			      ctxt->status->stream->timebase->t.sec);
+			      ctxt->status->stream->basetime->t.sec);
     } else if (cmml_strcmp(attr[i], "title") == 0) {
       m->title = cmml_strdup(attr[i+1]);
     }
@@ -452,6 +463,65 @@ insert_meta_tag(CMML_List * meta_list, const XML_Char **attr, CMML_Context *ctxt
 }
 
 
+/*
+ * insert_link_tag: Make a new CMML_LinkElement and add it to the list
+ * of CMML_LinkElements contained within the clip or head
+ * CMML_Element.
+ */
+static CMML_List * 
+insert_link_tag(CMML_List * link_list, const XML_Char **attr, CMML_Context *ctxt)
+{
+  int i;
+  CMML_LinkElement * l;
+
+  if (attr[0] == NULL) { /* no attributes present */
+    return NULL;
+  } 
+  
+  if ((l = (CMML_LinkElement *) cmml_malloc(sizeof(CMML_LinkElement)))
+      == NULL) return NULL;
+  l->id = NULL;
+  l->class = NULL;
+  l->title = NULL;
+  l->lang = NULL;
+  l->dir = NULL;
+  l->href = NULL;
+  l->type = NULL;
+  l->rel = NULL;
+  l->rev = NULL;
+  l->media = NULL;
+
+  for (i = 0; attr[i] != NULL; i += 2) {
+#ifdef DEBUG
+    fprintf(stderr, "Inserting link attribute %s=%s\n", attr[i], attr[i+1]);
+#endif
+    if (cmml_strcmp(attr[i], "id") == 0) {
+      l->id = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "class") == 0) {
+      l->class = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "title") == 0) {
+      l->title = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "lang") == 0) {
+      l->lang = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "dir") == 0) {
+      l->dir = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "href") == 0) {
+      l->href = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "type") == 0) {
+      l->type = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "rel") == 0) {
+      l->rel = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "rev") == 0) {
+      l->rev = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "media") == 0) {
+      l->media = cmml_strdup(attr[i+1]);
+    }
+  }
+  link_list = cmml_list_append(link_list, l);
+  return link_list;
+}
+
+
 /*
  * insert_clip_attrs: insert attributes from an "a" element into a
  * CMML_Element of type CMML_CLIP.
@@ -465,7 +535,7 @@ insert_clip_attrs(CMML_Clip * a, const XML_Char **attr, CMML_Context * ctxt)
   if (!ctxt->status->stream) {
     /* the document has no stream tag before this clip tag */
     ctxt->status->stream = cmml_stream_new();
-    ctxt->status->stream->timebase = cmml_time_new("0");
+    ctxt->status->stream->basetime = cmml_time_new("0");
   }
   for (i = 0; attr[i] != NULL; i += 2) {
 #ifdef DEBUG
@@ -473,6 +543,10 @@ insert_clip_attrs(CMML_Clip * a, const XML_Char **attr, CMML_Context * ctxt)
 #endif
     if (cmml_strcmp(attr[i], "id") == 0) {
       a->clip_id = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "class") == 0) {
+      a->class = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "title") == 0) {
+      a->title = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "lang") == 0) {
       a->lang = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "dir") == 0) {
@@ -483,12 +557,12 @@ insert_clip_attrs(CMML_Clip * a, const XML_Char **attr, CMML_Context * ctxt)
       a->start_time = 
 	cmml_time_new_in_sec (attr[i+1], 
 			      ctxt->status->stream->utc,
-			      ctxt->status->stream->timebase->t.sec);
+			      ctxt->status->stream->basetime->t.sec);
     } else if (cmml_strcmp(attr[i], "end") == 0) {
       a->end_time = 
 	cmml_time_new_in_sec (attr[i+1],
 			      ctxt->status->stream->utc,
-			      ctxt->status->stream->timebase->t.sec);
+			      ctxt->status->stream->basetime->t.sec);
     }
   }
   /* set default track  */
@@ -507,16 +581,16 @@ insert_clip_attrs(CMML_Clip * a, const XML_Char **attr, CMML_Context * ctxt)
     cmml_set_error((CMML*)ctxt, CMML_FORMAT_ERROR);
     cmml_time_free (a->end_time);
   }
-  /* check if new clip comes after timebase and after previous clip */
-  if (a->start_time->t.sec < ctxt->status->stream->timebase->t.sec ||
+  /* check if new clip comes after basetime and after previous clip */
+  if (a->start_time->t.sec < ctxt->status->stream->basetime->t.sec ||
       (ctxt->status->clip && 
        (a->start_time->t.sec < ctxt->status->clip->start_time->t.sec))) {
       cmml_set_error((CMML*)ctxt, CMML_NONSEQUENTIAL_CLIP);
 #ifdef DEBUG
       fprintf(stderr, 
-	      "nonsequential: clip=%f(%s), timebase=%f, prev_clip=%f\n",
+	      "nonsequential: clip=%f(%s), basetime=%f, prev_clip=%f\n",
 	      a->start_time->t.sec, a->start_time->tstr,
-	      ctxt->status->stream->timebase->t.sec,
+	      ctxt->status->stream->basetime->t.sec,
 	      ctxt->status->prev_clip->start_time->t.sec);
 #endif
       return -1;
@@ -548,6 +622,8 @@ insert_anchor_attrs(CMML_Clip * clip, const XML_Char **attr, CMML_Context *ctxt)
       clip->anchor_id = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "class") == 0) {
       clip->anchor_class = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "title") == 0) {
+      clip->anchor_title = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "lang") == 0) {
       clip->anchor_lang = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "dir") == 0) {
@@ -591,6 +667,10 @@ insert_img_attrs(CMML_Clip * clip, const XML_Char **attr, CMML_Context *ctxt)
 #endif
     if (cmml_strcmp(attr[i], "id") == 0) {
       clip->img_id = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "class") == 0) {
+      clip->img_class = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "title") == 0) {
+      clip->img_title = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "lang") == 0) {
       clip->img_lang = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "dir") == 0) {
@@ -636,6 +716,10 @@ insert_desc_attrs(CMML_Clip * clip, const XML_Char **attr, CMML_Context* ctxt)
 #endif
     if (cmml_strcmp(attr[i], "id") == 0) {
       clip->desc_id = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "class") == 0) {
+      clip->desc_class = cmml_strdup(attr[i+1]);
+    } else if (cmml_strcmp(attr[i], "title") == 0) {
+      clip->desc_title = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "lang") == 0) {
       clip->desc_lang = cmml_strdup(attr[i+1]);
     } else if (cmml_strcmp(attr[i], "dir") == 0) {
@@ -710,6 +794,9 @@ cmml_doctype_parse (void *userData,
  * If the tag name is "meta" then the CMML_Element at the tail of the
  * list can be either a head or an clip.
  *
+ * If the tag name is "link" then the CMML_Element at the tail of the
+ * list can be either a head or an clip.
+ *
  * The above arrangement takes advantage of the "shallowness" of the
  * CMML DTD, that is, whenever "title", "stream", "base", "desc",
  * "param" or "meta" is encountered, then its parent element is
@@ -768,10 +855,10 @@ cmml_start(void *cmml, const XML_Char *name, const XML_Char **attrs)
     /* save stream tag */
     context->status->stream = stream;
     /* if in the middle of seeking for a time offset,
-       and seektime is before timebase, stop seeking and report error */
+       and seektime is before basetime, stop seeking and report error */
     if (context->status->seek_type == _CMML_SEEK_TIME) {
       if (context->status->seek_secs <= 
-	  context->status->stream->timebase->t.sec) {
+	  context->status->stream->basetime->t.sec) {
 	context->status->seek_secs = _CMML_SEEK_FOUND;
 	context->status->seek_secs = -1.0;
       }
@@ -816,7 +903,7 @@ cmml_start(void *cmml, const XML_Char *name, const XML_Char **attrs)
     /* if there has been no stream tag, fix it up */
     if (!context->status->stream) {
       context->status->stream = cmml_stream_new();
-      context->status->stream->timebase = cmml_time_new("0");
+      context->status->stream->basetime = cmml_time_new("0");
     }
     context->status->type = CMML_HEAD;
     /* there musn't be a head tag after a clip tag */
@@ -890,6 +977,34 @@ cmml_start(void *cmml, const XML_Char *name, const XML_Char **attrs)
     }
     return;
 
+  } else if (cmml_strcmp(name, "link") == 0) {
+    if (context->status->type != CMML_HEAD
+/* this is in preparation for CMML 3.0
+        && context->status->type != CMML_CLIP
+ */
+      ) {
+      cmml_set_error(cmml, CMML_PARSE_ERROR);
+      return;
+    }
+    if (context->status->type == CMML_HEAD) {
+      if (context->status->head == NULL) {
+	cmml_set_error(cmml, CMML_PARSE_ERROR);
+	return;
+      }
+      context->status->head->link =
+	insert_link_tag(context->status->head->link, attrs, cmml);
+    } else { /* CMML_CLIP */
+/* this is in preparation for CMM 3.0
+      if (context->status->clip == NULL) {
+	cmml_set_error(cmml, CMML_PARSE_ERROR);
+	return;
+      }
+      context->status->clip->link =
+	insert_link_tag(context->status->clip->link, attrs, cmml);
+ */
+    }
+    return;
+
   /* ----- the "clip" tag ----- */
   } else if (cmml_strcmp(name, "clip") == 0) {
     /* has to be after cmml and head */
@@ -990,7 +1105,7 @@ clip_in_interval(CMML_Context * ctxt)
       ctxt->status->t_start = 
 	cmml_time_utc_to_sec (ctxt->status->t_start,
 			      ctxt->status->stream->utc,
-			      ctxt->status->stream->timebase->t.sec);
+			      ctxt->status->stream->basetime->t.sec);
       if (ctxt->status->t_start->t.sec < 0) {
 	cmml_time_free(ctxt->status->t_start);
 	ctxt->status->t_start = NULL;
@@ -1012,7 +1127,7 @@ clip_in_interval(CMML_Context * ctxt)
       ctxt->status->t_end = 
 	cmml_time_utc_to_sec (ctxt->status->t_end,
 			      ctxt->status->stream->utc,
-			      ctxt->status->stream->timebase->t.sec);
+			      ctxt->status->stream->basetime->t.sec);
       if (ctxt->status->t_end->t.sec < 0) {
 	cmml_time_free(ctxt->status->t_end);
 	ctxt->status->t_end = NULL;
@@ -1146,6 +1261,9 @@ cmml_end(void *cmml, const XML_Char *name)
   } else if (cmml_strcmp(name, "meta") == 0) {
     /* empty tag: no action */
     return;
+  } else if (cmml_strcmp(name, "link") == 0) {
+    /* empty tag: no action */
+    return;
 
   /* ----- the "clip" tag ----- */
   } else if (cmml_strcmp(name, "clip") == 0) {
@@ -1253,6 +1371,7 @@ CMML_Context * cmml_context_setup (FILE * file)
   context->status->info->cmml_dir         = NULL;
   context->status->info->cmml_id          = NULL;
   context->status->info->cmml_xmlns       = NULL;
+  context->status->info->cmml_granulerate = NULL;
   context->status->type = CMML_NONE;
   context->status->stream = NULL;
   context->status->import_cnt = -1;
